package com.synclovis.trackflow;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;

import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Promise;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.synclovis.mytrackflowlibrary.TrackflowLibraryModule;
import com.synclovis.mytrackflowlibrary.connector.Client;
import com.synclovis.mytrackflowlibrary.utils.LibConstants;
import com.synclovis.mytrackflowlibrary.utils.LibTFUtils;
import com.synclovis.mytrackflowlibrary.utils.ServicesOptions;
import com.synclovis.trackflow.connectors.Trackflow;
import com.synclovis.trackflow.utils.TFTrackingConstants;
import com.synclovis.trackflow.utils.TFUtils;
import com.synclovis.trackflow.utils.TrackingOptions;

import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Build;
import android.widget.Toast;
import android.content.Context;
import android.view.View;

import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Map;

import static android.Manifest.permission.ACCESS_COARSE_LOCATION;
import static android.Manifest.permission.ACCESS_FINE_LOCATION;

import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableType;



public class TrackflowModule extends ReactContextBaseJavaModule implements ActivityCompat.OnRequestPermissionsResultCallback,
  LifecycleEventListener {

  public static TrackingOptions TrackOptions;
  private final ReactApplicationContext reactContext;
  private Context context;
  String appPackageName = getReactApplicationContext().getPackageName();
  String[] permissions = {"android.permission.FINE_LOCATION"};
  Activity activity = getCurrentActivity();
  int permsRequestCode = 200;


  public TrackflowModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
    this.context = reactContext.getApplicationContext();

    reactContext.addLifecycleEventListener(this);
  }

  /**
   * startTracking
   * This method is used start tracking of all the features available in Trackflow
   *
   * @param apiKey Pass APIKEY obtained after creating project in https://wwww.trackflow.ai
   */
  @RequiresApi(api = Build.VERSION_CODES.O)
  @ReactMethod
  public void startTracking(String apiKey) {
    //LibConstants.SHAKE_SERVICE = true;
    LibConstants.SCREENSHOT_SERVICE = true;
    LibConstants.PERMISSION_SERVICE = true;
    LibConstants.NETWORK_SERVICE = true;
    LibConstants.SHAREDPREF_SERVICE = true;
    LibConstants.DEVICEINFO_SERVICE = true;
    LibConstants.SCREEN_NAVIGATION = true;
    Trackflow trackflow = new Trackflow(apiKey, getReactApplicationContext(), getCurrentActivity());
    // TrackflowLibraryModule aarLib = new TrackflowLibraryModule(apiKey,getReactApplicationContext(),getCurrentActivity());
    trackflow.startTracking();

    /*
     *This method is used when parent project doesn't have location permission,and the user wants to enable location tracking
     */
    //trackflow.askLocationPermission();
  }

  @RequiresApi(api = Build.VERSION_CODES.O)
  @ReactMethod
  public void startTrackingServices(String apiKey, ReadableArray value ) {
  for (int i = 0; i < value.size(); i++) {

      String service = value.getString(i);
      switch (service) {
        case "SHAKE":
        LibConstants.SHAKE_SERVICE = true;
          break;
        case "SCREENSHOT_WEB_REQUEST":
          LibConstants.SCREENSHOT_SERVICE = true;
          break;
        case "PERMISSION":
          LibConstants.PERMISSION_SERVICE = true;
          break;
        case "NETWORKLOGS":
          LibConstants.NETWORK_SERVICE = true;
          break;
        case "ASYNCSTORAGE":
          LibConstants.SHAREDPREF_SERVICE = true;
          break;
        case "DEVICEINFO":
          LibConstants.DEVICEINFO_SERVICE = true;
          break;
        case "AUTO_CAPTURE":
          LibConstants.SCREEN_NAVIGATION = true;
          break;
        default:
          break;
      }
    }
    Trackflow trackflow = new Trackflow(apiKey, getReactApplicationContext(), getCurrentActivity());
    //   // TrackflowLibraryModule aarLib = new TrackflowLibraryModule(apiKey,getReactApplicationContext(),getCurrentActivity());
    trackflow.startTracking();
    /*
     *This method is used when parent project doesn't have location permission,and the user wants to enable location tracking
     */
    //trackflow.askLocationPermission();
  }


  @ReactMethod
  public void analyse(String screenName) {
   // TFUtils.showLog("====="+screenName); //analyse
    Trackflow trackflow = new Trackflow(getReactApplicationContext(), getCurrentActivity());

    trackflow.screenNavigateCapture(false,screenName);
  }

  @RequiresApi(api = Build.VERSION_CODES.O)
  @ReactMethod
  public void log(String log) {
    Client.getInstance().sendLog(log);
 }

  @Override
  public Map<String, Object> getConstants() {
    final Map<String, Object> constants = new HashMap<>();
    constants.put("SHAKE", TrackingOptions.SHAKE);
    constants.put("NETWORKLOGS", TrackingOptions.networklogs);
    constants.put("PERMISSION", TrackingOptions.permission);
    constants.put("DEVICEINFO", TrackingOptions.deviceInfo);
    constants.put("AUTO_CAPTURE", TrackingOptions.autoScreenshotCapture);
    constants.put("SCREENSHOT_WEB_REQUEST", TrackingOptions.screenshotRequest);
    constants.put("ASYNCSTORAGE", TrackingOptions.asyncStorage);


    return constants;
  }

  @RequiresApi(api = Build.VERSION_CODES.O)
  @ReactMethod
  public void getDeviceSpecification(Promise p) {

    Trackflow trackflow = new Trackflow(getReactApplicationContext(), getCurrentActivity());
    trackflow.deviceSpecification(p);
  }


  @RequiresApi(api = Build.VERSION_CODES.M)
  public void requestRequiredPermission() {
    ActivityCompat.requestPermissions(activity, new String[]{ACCESS_FINE_LOCATION, ACCESS_COARSE_LOCATION}, permsRequestCode);
  }


  @ReactMethod
  public void logNetwork(String log) {
    // TFUtils.showLog("log isssss"+log);
    Trackflow trackflow = new Trackflow(getReactApplicationContext());
    trackflow.sendNetworkLogs(log);
  }

  @ReactMethod
  public void logAsyncStorage(String log) {
    Trackflow trackflow = new Trackflow(getReactApplicationContext());
    // TrackflowLibraryModule aarLib = new TrackflowLibraryModule(getReactApplicationContext(),getCurrentActivity());

    trackflow.sendAsyncLogs(log);
  }

  @Override
  public String getName() {
    return "Trackflow";
  }

  @Override
  public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {

    switch (requestCode) {
      case 200:
        // If request is cancelled, the result arrays are empty.
        if (grantResults.length > 0 &&
          grantResults[0] == PackageManager.PERMISSION_GRANTED) {

        }
    }
  }

  @Override
  public void onHostResume() {
    //LibTFUtils.showLog("jiJIOJIOJIOJ-->"+getCurrentActivity());
//    getReactApplicationContext()
//      .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
//      .emit("activityResume", null);
  }

  @Override
  public void onHostPause() {
//    getReactApplicationContext()
//      .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
//      .emit("activityPause", null);
  }

  @Override
  public void onHostDestroy() {
  }
}
