/*
 * Copyright (c) 2020. Synclovis Systems Pvt Ltd - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */

package com.synclovis.trackflow.connectors;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.hardware.SensorManager;
import android.os.Build;

import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.WritableNativeMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.synclovis.mytrackflowlibrary.TrackflowLibraryModule;
import com.synclovis.mytrackflowlibrary.connector.Client;
import com.synclovis.mytrackflowlibrary.connector.SocketListener;
import com.synclovis.mytrackflowlibrary.utils.LibConstants;
import com.synclovis.mytrackflowlibrary.utils.LibTFUtils;
import com.synclovis.mytrackflowlibrary.utils.ServicesOptions;
import com.synclovis.trackflow.deviceShakeListener.SensorDetector;
import com.synclovis.trackflow.deviceShakeListener.ShakeListener;
import com.synclovis.trackflow.utils.Constants;
import com.synclovis.trackflow.utils.TFUtils;

import org.json.JSONException;
import org.json.JSONObject;

import androidx.annotation.*;

import android.location.LocationManager;
import android.os.Bundle;
import android.view.View;
import android.widget.Toast;

import java.util.concurrent.ExecutionException;

/**
 * Trackflow - Tracking Manager
 *
 * @author Anooj Krishnan G <anooj.krishnan@synclovis.com>
 * @author Nousheen K <nousheen.k@synclovis.com>
 * @version 1.0
 */


public class Trackflow extends Application implements SocketListener, ShakeListener {

  private String apiKey;
  private Context mcontext;
  private ReactApplicationContext context;
  private PackageInfo packageInfo;
  private Activity activity;
  private LocationManager locationManager;
  private Object Promise;
  private Object ReadableMap;
  static String screenName = "";
  private SensorDetector mDetector;
  private SensorManager mSensorManager;

  public Trackflow(String apiKey, ReactApplicationContext context, Activity activity) {
    this.context = context;
    this.apiKey = apiKey;
    this.activity = activity;
    this.mcontext = context.getApplicationContext();
    this.initializeLocationManager();
  }

  public Trackflow(ReactApplicationContext context) {
    this.context = context;
  }

  public Trackflow(ReactApplicationContext context, Activity activity) {
    this.mcontext = context.getApplicationContext();
    this.activity = activity;
  }

  private void initializeLocationManager() {
    try {
      locationManager = (LocationManager) context.getApplicationContext().getSystemService(Context.LOCATION_SERVICE);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }


  @RequiresApi(api = Build.VERSION_CODES.O)
  public void startTracking() {
    establishConection();

  }



  @RequiresApi(api = Build.VERSION_CODES.O)
  public void establishConection() {
    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);
    JSONObject metaInfo = aarLib.metaInfo();
    Client.getInstance().connectSocket(apiKey, metaInfo, this);
    //Device Shake listener
    if (LibConstants.SHAKE_SERVICE) {
    //  LibTFUtils.showLog("enterd in shake");
      mDetector = new SensorDetector(this);
      startListening(context.getApplicationContext());
    }
  }


  private void sendPermissionsList() {
    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);
    JSONObject permissionString = aarLib.permssionList();
    Client.getInstance().sendPermission(permissionString);
  }

  //device identifier app details
  @RequiresApi(api = Build.VERSION_CODES.O)
  public void deviceSpecification(Promise p) {
    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);
    JSONObject deviceSpztn = aarLib.deviceDetails();

    try {
      WritableMap map = new WritableNativeMap();
      WritableMap deviceSpztnMap = TFUtils.convertJsonToMap(deviceSpztn);
      map.putMap(Constants.DEVICE_SPECIFICATION, deviceSpztnMap);
      p.resolve(map);
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }


  public void sendScreenshot(boolean screenshot, String screenName) {
    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);
    String base64String = aarLib.getScreenShotBase64();
    if (base64String != null) {
      try {
        if (screenshot) {
          Client.getInstance().sendScreenshot(base64String, screenName);
        } else {
          Client.getInstance().screenNavigation(base64String, screenName);
        }

      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  @RequiresApi(api = Build.VERSION_CODES.M)
  private void sendDiviceInitDetails() {
    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);
    JSONObject deviceInitDetails = aarLib.onDeviceInfo();
    Client.getInstance().deviceDetail(deviceInitDetails);
  }

  private void sendEvent(String eventName, String params) {
    context.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(eventName, params);
  }

  public void sendNetworkLogs(String log) {
    Client.getInstance().sendNetworkLogs(log);
  }

  public void sendAsyncLogs(String log) {
    Client.getInstance().sendAsyncLogs(log);
  }


  @RequiresApi(api = Build.VERSION_CODES.N)
  @Override
  public void onConnectionSuccess() {

    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);
    JSONObject deviceInitDetails = aarLib.deviceInitDetails();
    Client.getInstance().deviceInfo(deviceInitDetails);
  }

  @Override
  public void onConnectionFailed() {

  }

  @Override
  public void onPermissionRequest() {
    sendPermissionsList();
  }

  @Override
  public void onScreenShotRequest() {
    sendScreenshot(true, "screenNameDummy");
  }

  @RequiresApi(api = Build.VERSION_CODES.M)
  @Override
  public void onDeviceInfo() {
    sendDiviceInitDetails();
  }

  @Override
  public void onAsyncRequest() {

    sendEvent("async_storage", "params");
  }

  @Override
  public void startListening(Context context) {
    // LibTFUtils.showLog("shake entered in listener");
    mSensorManager = (SensorManager) context.getSystemService(Context.SENSOR_SERVICE);
    mDetector.start(mSensorManager);
  }

  /**
   * Shake detected do the following work in order
   *
   * <li> Get root view </li>
   * <li> Take screenshot of current context </li>
   * <li> Start feedback acitivty </li>
   */
  @Override
  public void onDeviceShaked() throws InterruptedException, ExecutionException {
    onShakeDevice("React_Native");
   // Toast.makeText(context, "Screenshot captured", Toast.LENGTH_LONG).show();
  }

  /**
   * Context will be set to null
   * and {@link SensorManager} will be released
   */
  @Override
  public void stopListening(Context context) {
    this.context = null;
    mDetector.stop();
  }

  @Override
  public void onApiCall() {
    screenNavigateCapture(false,"onApi");
  }

  public void screenNavigateCapture(boolean scrrenshot, String screenName) {
    // if(LibConstants.SCREEN_NAVIGATION) {
    //   sendScreenshot(false,"sendNavigationScreen");
    // }
    if (this.activity != null) {
     // TFUtils.showLog("screenName activirt");
      final View view = this.activity.getWindow().getDecorView().getRootView();
      if (view == null) {
       // LibTFUtils.showLog("entered in view null");
        // return null;
      }
      view.setDrawingCacheEnabled(true);
      view.post(new Runnable() {
        @Override
        public void run() {
        //  TFUtils.showLog("in run-->" + screenName);
          sendScreenshot(scrrenshot, screenName);
        }
      });
    } else {
     // TFUtils.showLog("=====+in act null screenName");
    }
  }
  public void onShakeDevice(String screenName) {
    TrackflowLibraryModule aarLib = new TrackflowLibraryModule(this.mcontext, this.activity);

    if (this.activity != null) {
      // TFUtils.showLog("screenName activirt");
      final View view = this.activity.getWindow().getDecorView().getRootView();
      if (view == null) {
        // LibTFUtils.showLog("entered in view null");
        // return null;
      }
      view.setDrawingCacheEnabled(true);
      view.post(new Runnable() {
        @Override
        public void run() {
          //  TFUtils.showLog("in run-->" + screenName);
          String base64String = aarLib.getScreenShotBase64();
          if (base64String != null) {
            try {

                Client.getInstance().shakeDevice(base64String, screenName);


            } catch (Exception e) {
              e.printStackTrace();
            }
          }
        }
      });
    } else {
      // TFUtils.showLog("=====+in act null screenName");
    }
  }

}
