//
//  TFShakeListener.swift
//  Trackflow
//
//  Created by Jyothi Menda on 26/03/21.
//  Copyright © 2021 Facebook. All rights reserved.
//


import Foundation
import CoreMotion
import UIKit
import Trackflow


class TFShakeListener:NSObject{
    
    var coreMotionManager: CMMotionManager!
    public static let shared = TFShakeListener();
    private var lastShakedOn = Date();
    
    override init() {
        super.init()
    }
    
    public func start(){
        coreMotionManager = CMMotionManager()
        self.getDeviceMotion(coreMotionManager : coreMotionManager, withEdit: false)
    }
    
    // public func start(withEdit:Bool){
    //     coreMotionManager = CMMotionManager()
    //     self.getDeviceMotion(coreMotionManager : coreMotionManager, withEdit: true)
    // }
    
    private func getDeviceMotion(coreMotionManager : CMMotionManager, withEdit:Bool) {
        if coreMotionManager.isAccelerometerAvailable {
            coreMotionManager.accelerometerUpdateInterval = 1/3;
            coreMotionManager.startAccelerometerUpdates(to: .main) {
                (data, error) in
                guard let data = data, error == nil else {
                    return
                }
                let valueX = fabs(data.acceleration.x);
                let valueY = fabs(data.acceleration.y);
                let valueZ = fabs(data.acceleration.z);
                
                var maxValue = valueX > valueY ? valueX : valueY;
                maxValue = maxValue > valueZ ? maxValue : valueZ;
                
                let kThreshold  = 1.6;
                
                if (maxValue > kThreshold) {
                    let currentShakeTime = Date();
                    
                    let formatter = DateComponentsFormatter()
                    formatter.allowedUnits = [.second]
                    let differenceInString = formatter.string(from: self.lastShakedOn, to: currentShakeTime)!
                    let difference = Int(differenceInString)!;
                    
                    if(difference > 1){
                        let capturedImage = TFCaptureManager.shared.capture();
                        Transporter.shared.sendShakeCapturedImageData(data: capturedImage, scName: "TestScreen")
                        self.lastShakedOn = currentShakeTime;
//                        if(withEdit){
//                            let vc = ImageEditorController();
//                            vc.base64Img = capturedImage;
//                            UIApplication.shared.keyWindow?.rootViewController?.present(vc, animated: true, completion: nil)
//                        }
                        }
                    }
                }
             }
        
        
    }
    
 
}
