//
//  Transporter.swift
//  react-native-trackflow
//
//  Created by Anooj Krishnan G on 06/10/20.
//

import Foundation
import SocketIO
import Trackflow

class Transporter : NSObject {
    public static let shared = Transporter();
    var socket:SocketIOClient?;
    var socketManager:SocketManager?;
    var isLogEnabled:Bool = false;
    var forecPolling:Bool = true;
    let TAG = "Trackflow.AI"
    var timer:Timer?;
    var timerTriggerDate:Date?;
    
    

   override init() {
            super.init()
            
        }
        
        public func connect(url:String, params:[String:String]){
     
         let _url = URL(string: url)!
            socketManager = SocketManager(socketURL: _url, config: [.log(isLogEnabled), .forcePolling(forecPolling), .connectParams(params),.compress]);
            
            self.socket = self.socketManager?.socket(forNamespace: "/socket.io");
            socket?.connect();
            listenEvents();
        }
        
        func disconnect(){
            socket?.disconnect();
        }
        
        func getConnection() -> SocketIOClient {
            return socket!;
        }
        
        public func isConnected() -> Bool{
            return socket?.status.active ?? false;
        }
        
        func listenEvents(){
            socket?.on(TFConstants.SOCKET_ON_CONNECT){ dataArray, ack in
                //print(self.TAG," Connection Successfull",dataArray)
                self.socket?.emit(TFConstants.SOCKET_SEND_DEVICE_INIT, TFManager.shared.getInitInfo());
            }
            
            socket?.on(TFConstants.SOCKET_ON_ACK){ dataArray, ack in
                //print(self.TAG," acknowledge_",dataArray)
                let ackObj = dataArray[0] as! NSDictionary;
                let ackEvent = ackObj["event"] as! String;
                
                switch ackEvent {
                    case TFConstants.SOCKET_SEND_DEVICE_INIT:
                        //print("DeviceINIT")
                        //self.initTimer()
                        break;
                    case TFConstants.SOCKET_SEND_LOG_INFO:
                        //print("Log Sent!!")
                        break;
                    default:
                        break;
                }
            }
            
            
            socket?.on("error"){ dataArray, ack in
                                
            }
            
            socket?.on(TFConstants.SOCKET_ON_ERROR){ dataArray, ack in
                do{
                    let errorObj = dataArray[0] as! NSDictionary;
                    let err = errorObj["error"] as! String;
                    TFLog.shared.showErrorLog(errorMsg: err);
                } catch {
                    
                }
                
            }
            
            
            socket?.on(TFConstants.SOCKET_ON_SCREENSHOTREQ){ dataArray, ack in
                
            let screenshot = TFCaptureManager.shared.capture();
                self.sendCapturedImageData(data: screenshot);
                
            }
            
            socket?.on(TFConstants.SOCKET_ON_ASYNCSTORAGEREQ){ dataArray, ack in   
            if TFConstants.isAsyncStorageReq {
                EventEmitter.sharedInstance.dispatch(name: "async_storage", body: "async_storage")

            }             
            }
            
            socket?.on(TFConstants.SOCKET_ON_DEVICEINFOREQ){ dataArray, ack in   
            if TFConstants.isDeviceInfoReq {
            if(self.isConnected()){
                    self.socket?.emit(TFConstants.SOCKET_SEND_DEVICEINFO, TFManager.shared.initDeviceInfo());
                }
            }             
               
            }
            
            socket?.on(TFConstants.SOCKET_ON_PERMISSIONREQ){ dataArray, ack in  
             if TFConstants.isPermissionsEnabled{
                 if(self.isConnected()){
                    self.socket?.emit(TFConstants.SOCKET_SEND_PERMISSION, TFPermissionManager.shared.getPermissionList());
                }
             }              
              
            }
            
            
        }
        
        func initTimer(){
            if #available(iOS 10.0, *) {
                
                if(timer == nil){
                    timerTriggerDate = Date();
                    timer = Timer.scheduledTimer(withTimeInterval: 60.0, repeats: true) { timer in
                        let now:Date = Date();                        
                        if((TFUtils.shared.getMinutesDifferenceFromTwoDates(start: self.timerTriggerDate!, end: now)) % 10 == 0){
                            self.fetchPermissionLog();
                        }
                        
                    }
                }
                
            }
        }
        
          func sendNetworkLog(log:String){

            if TFConstants.isAPILogEnabled {
                if(isConnected()){
                socket?.emit(TFConstants.SOCKET_SEND_LOG_INFO, log);
                }
            }
            
             if TFConstants.isAutomaticScreenshotEnabled{
                 DispatchQueue.main.async { 
                 let screenshot = TFCaptureManager.shared.capture();
                 self.socket?.emit(TFConstants.SOCKET_AUTO_CAPTURE,screenshot,"REACT_NATIVE");
                }
             }
            }
    
        func sendAsyncStorage(log:String){
                
                if TFConstants.isAsyncStorageReq {
                if(isConnected()){
                    socket?.emit(TFConstants.SOCKET_SEND_ASYNCSTORAGE, log);
                }
                }
            }

            func sendCapturedImageData(data:String){
                          if TFConstants.isScreenShotReqEnabled {
                           if(isConnected()){
                socket?.emit(TFConstants.SOCKET_SEND_SCREENSHOT,data,"","REACT_NATIVE");
           }
                        }
              
            }
    
        func autoCapturedImageData(screenName:String){
                    if TFConstants.isAutomaticScreenshotEnabled {
                    DispatchQueue.main.async { 
                        let screenshot = TFCaptureManager.shared.capture();
                        if(self.isConnected()){
                                self.socket?.emit(TFConstants.SOCKET_AUTO_CAPTURE,screenshot,screenName);
                                }
                            }
                        }
                    }
  
                func shakeToCaptureData(data:String, scName:String){
                            if TFConstants.isShakeDeviceEnabled   {
                            if(isConnected()){ 
                            socket?.emit(TFConstants.SOCKET_SHAKE_DEVICE,data , scName);

            }
                            }
                
                }

            func listPermissionEvent(){
            }

            func fetchPermissionLog(){
                let permissionList = TFPermissionManager.shared;
                let permList = permissionList.getPermissionList();
                let permissionObj: NSMutableDictionary     = NSMutableDictionary();
                permissionObj.setValue(permList, forKey: "permissions")
        
                if TFConstants.isPermissionsEnabled {
        
                if(isConnected()){
                    socket?.emit("device_permissions",permissionObj);
                }
                }

            }

            func listenToLocation(){
             }

             func listenMemoryUsageEvent(){
             }

             func sendMemoryUsageData(data:String){
             }

            func sendLog(log:String){
                if(isConnected()){ 
                        socket?.emit("custom_log",log);
              //  socket?.emit(TFConstants.CUSTOM_LOG,data , scName);
              }
                }

            func sendShakeCapturedImageData(data:String, scName:String){

                if(isConnected()){
                    socket?.emit("shake_to_screenshot", data , scName);
                }
            }
        
    
    
}
