import XHRInterceptor from 'react-native/Libraries/Network/XHRInterceptor';
import { string } from 'prop-types';
import { NativeModules } from 'react-native';
const { Trackflow } = NativeModules;

let nextCallId = 0;
let requests = [];
let requestObj = {}

export default new class Logger {     

    enableXHRInterception(o) {        
        if (XHRInterceptor.isInterceptorEnabled()) {
          return;
        }    
            
        XHRInterceptor.setOpenCallback(this.openCallback);
        XHRInterceptor.setRequestHeaderCallback(this.requestHeadersCallback);
        XHRInterceptor.setHeaderReceivedCallback(this.headerReceivedCallback);
        XHRInterceptor.setSendCallback(this.sendCallback);
        XHRInterceptor.setResponseCallback(this.responseCallback);    
        XHRInterceptor.enableInterception();    
      }

    openCallback(method, url, xhr) {
        xhr._callid = nextCallId++;
        xhr._startTime = new Date()    
        let req = {url:url,method:method}
        requestObj[xhr._callid]=req
        requests.push(req)
    }

    requestHeadersCallback(header, value, xhr) {        
        let prevHeader = {}
        prevHeader[header] = value;
        if(requestObj[xhr._callid] != undefined && requestObj[xhr._callid].header != undefined){
            requestObj[xhr._callid].header = {...requestObj[xhr._callid].header, ...prevHeader}
        }else{
            requestObj[xhr._callid].header = {...prevHeader}
        } 
        requests[xhr._callid] = requestObj[xhr._callid]     
    }

    headerReceivedCallback(responseContentType, responseSize, responseHeaders, xhr){        
        if(requestObj[xhr._callid] != undefined && requestObj[xhr._callid] != null){
            requestObj[xhr._callid].responseHeaders = xhr.responseHeaders
            requestObj[xhr._callid].responseSize = responseSize
            requestObj[xhr._callid].responseContentType = responseContentType
        }

    }

    sendCallback(data, xhr){        
        if(requestObj[xhr._callid] != undefined && requestObj[xhr._callid] != null){
            requestObj[xhr._callid].body = data
            requestObj[xhr._callid].startTime = new Date()
            requests[xhr._callid] = requestObj[xhr._callid]
        }
    }

    responseCallback(status,timeout, response, responseURL, responseType, xhr){       
        var _response = response;
        if(responseType === 'blob' && typeof FileReader !== 'undefined' && responseType){
            const bReader = new FileReader()
            let self = this;
            const brListener = () => {                
                requestObj[xhr._callid].status = status
                requestObj[xhr._callid].timeout = timeout
                requestObj[xhr._callid].response = bReader.result
                requestObj[xhr._callid].responseType = responseType
                requestObj[xhr._callid].endTime = new Date()
                requests[xhr._callid] = requestObj[xhr._callid]            
               
                Logger.prototype.sendMetrics(requestObj[xhr._callid], xhr._callid)

                bReader.removeEventListener('loadend', brListener)
            }
            bReader.addEventListener('loadend', brListener)
            bReader.readAsText(response)
        } else if(requestObj[xhr._callid] != undefined){
        
            requestObj[xhr._callid].status = status
            requestObj[xhr._callid].timeout = timeout
            requestObj[xhr._callid].response = response
            requestObj[xhr._callid].responseType = responseType
            requestObj[xhr._callid].endTime = new Date()
            requests[xhr._callid] = requestObj[xhr._callid]            
            
           
            Logger.prototype.sendMetrics(requestObj[xhr._callid], xhr._callid)
        }
    }

    sendResponse(status, timeout, response, responseType, xhr){
        requestObj[xhr._callid].status = status
        requestObj[xhr._callid].timeout = timeout
        requestObj[xhr._callid].response = response
        requestObj[xhr._callid].responseType = responseType
        requestObj[xhr._callid].endTime = new Date()
        requests[xhr._callid] = requestObj[xhr._callid]            
        
        Logger.prototype.sendMetrics(requestObj[xhr._callid], xhr._callid)
    }

    sendMetrics(reqObj, index){
        try{
        let raw = reqObj
        try{
            raw = reqObj.method == 'HEAD'?"{}":JSON.stringify(reqObj)
        }catch(e){
            raw = "{}"
        }

        let now = new Date();

        let ob = {
                duration: reqObj.timeout,
                status: reqObj.status,
                url: reqObj.url,
                method:reqObj.method,
                duration:reqObj.endTime.getTime() - reqObj.startTime.getTime(),                
                body:reqObj.body,
                response:reqObj.method != 'HEAD'? reqObj.response:"{}",                
                headers:reqObj.header,
                responseHeaders:reqObj.responseHeaders,
                responseSize:reqObj.responseSize,
                raw:reqObj.method != 'HEAD'?reqObj:{},
                startTime:reqObj.startTime!=undefined?reqObj.startTime:""+now,
                responseContentType:reqObj.responseContentType!=undefined?reqObj.responseContentType:"",
                _callid:index
        }        
        if(ob.url.indexOf("8081/symbolicate") == -1){
            Trackflow.logNetwork(JSON.stringify(ob));
        }        
        delete requestObj[index]
    }catch(e){
        
    }
    }
}